@@ -12,18 +12,13 @@ pac_size <- function(pac, lib.loc = .libPaths()) {
   stopifnot((length(pac) == 1) && is.character(pac))
   stopifnot(is.null(lib.loc) || (all(lib.loc %in% .libPaths()) && (length(list.files(lib.loc)) > 0)))
   stopifnot(pac %in% rownames(installed_packages(lib.loc = lib.loc)))
-  found <- try(find.package(pac, lib.loc = lib.loc), silent = TRUE)
-  if (inherits(found, "try-error")) {
-    0
-  } else {
-    dir_size(found)
-  }
+  dir_size(find.package(pac, lib.loc = lib.loc))
 }
 
 #' True size of the package
 #' @description True size of the package as it takes into account its all dependencies, recursively.
 #' @param pac character a package name.
-#' @param fields a character vector listing the types of dependencies, a subset of c("Depends", "Imports", "LinkingTo", "Suggests", "Enhances").
+#' @param fields a character vector listing the types of dependencies, a subset of `c("Depends", "Imports", "LinkingTo", "Suggests", "Enhances")`.
 #' Character string "all" is shorthand for that vector, character string "most" for the same vector without "Enhances", character string "strong" (default) for the first three elements of that vector.
 #' Default: `c("Depends", "Imports", "LinkingTo")`
 #' @param lib.loc character vector. Default: `.libPaths()`
@@ -59,7 +54,7 @@ pac_true_size <- function(pac,
 #' The app dependencies packages are checked recursively, and only in local repository.
 #' The default arguments setup is recommended.
 #' @param path path to the shiny app. Default: `"."`
-#' @param fields a character vector listing the types of dependencies, a subset of c("Depends", "Imports", "LinkingTo", "Suggests", "Enhances").
+#' @param fields a character vector listing the types of dependencies, a subset of `c("Depends", "Imports", "LinkingTo", "Suggests", "Enhances")`.
 #' Character string "all" is shorthand for that vector, character string "most" for the same vector without "Enhances", character string "strong" (default) for the first three elements of that vector.
 #' Default: `c("Depends", "Imports", "LinkingTo")`
 #' @param lib.loc character vector, used optionally when local is equal TRUE. Default: `.libPaths()`
