@@ -22,11 +22,8 @@ test_that("pacs::compareVersionsMin", {
 
 test_that("expand_dependency", {
   expect_identical(expand_dependency("strong"), c("Depends", "Imports", "LinkingTo"))
+  expect_identical(expand_dependency("all"), c("Depends", "Imports", "LinkingTo", "Suggests", "Enhances"))
+  expect_identical(expand_dependency("most"), c("Depends", "Imports", "LinkingTo", "Suggests"))
   expect_identical(expand_dependency(c("Depends", "Imports", "LinkingTo")), c("Depends", "Imports", "LinkingTo"))
-})
-
-test_that("pac_islast", {
-  skip_if_offline()
-  expect_identical(pac_islast("WRONG"), FALSE)
-  expect_true(is.logical(pac_islast("memoise")))
+  expect_error(expand_dependency("WRONG"))
 })
